/***************************************************************************
 *   Copyright (C) 2004 by Alexander Dymo                                  *
 *   adymo@kdevelop.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "dockwidget.h"

#include <qpair.h>
#include <qlayout.h>
#include <qcursor.h>
#include <qapplication.h>
#include <qobjectlist.h>
#include <qobjectdefs.h>

#include <kdebug.h>

#include "dockarea.h"
#include "dockheader.h"
#include "dockheaderbutton.h"
// #include "dockresizehandle.h"

namespace Ideal {

//struct DockInfo

DockInfo::DockInfo()
{
    geometry = QRect(0,0,0,0);
    layoutInfo.row = 0;
    layoutInfo.col = 0;
    dockPosition = 0;
}

QString DockInfo::write()
{
    return QString("%1 %2 %3 %4 %5").arg(geometry.x()).arg(geometry.y()).arg(geometry.width()).arg(geometry.height()).
        arg(layoutInfo.row).arg(layoutInfo.col).arg(dockPosition);
}

DockInfo DockInfo::read(const QString &str)
{
    DockInfo info;
    QStringList list = QStringList::split(" ", str, false);
    info.geometry.setX(list[0].toInt());
    info.geometry.setY(list[1].toInt());
    info.geometry.setWidth(list[2].toInt());
    info.geometry.setHeight(list[3].toInt());
    info.layoutInfo.row = list[4].toInt();
    info.layoutInfo.col = list[5].toInt();
    info.dockPosition = list[6].toInt();
    return info;
}




//class DockWidget


DockWidget::DockWidget(DockArea *area, Place place, const char *name)
    :QWidget(area, name), m_widget(0), m_docked(true), m_overlapped(true), m_area(area), m_place(place), m_oldArea(0)
{
    init();
}

DockWidget::DockWidget(const char *name)
    :QWidget(0, name), m_widget(0), m_docked(false), m_overlapped(true), m_area(0), m_place(Ideal::Bottom), m_oldArea(0)
{
    init();
    undock();
}

void DockWidget::init()
{
    m_layout = new QGridLayout(this, 2, 3, 0, 0);
    m_headerLayout = new QHBoxLayout(0);
    m_layout->addMultiCellLayout(m_headerLayout, 0, 0, 0, 2);
    m_header = new DockHeader(this);
    m_headerLayout->addWidget(m_header);
    
    m_detachOrDockButton = new DockHeaderButton(DockHeaderButton::Detach, this);
    m_overlapButton = new DockHeaderButton(DockHeaderButton::Overlap, this);
    m_closeButton = new DockHeaderButton(DockHeaderButton::Close, this);

    m_headerLayout->addWidget(m_detachOrDockButton);
    m_headerLayout->addWidget(m_overlapButton);
    m_headerLayout->addWidget(m_closeButton);
    
    m_detachOrDockButton->setOn(m_docked);
    m_overlapButton->setOn(m_overlapped);
    
    connect(m_detachOrDockButton, SIGNAL(clicked()), this, SLOT(detachOrDock()));
    connect(m_overlapButton, SIGNAL(clicked()), this, SLOT(overlap()));
    connect(m_closeButton, SIGNAL(clicked()), this, SLOT(closeDock()));
    
/*    leftHandle = new DockResizeHandle(Vertical, this);
    m_layout->addWidget(leftHandle, 1, 0);
    rightHandle = new DockResizeHandle(Vertical, this);
    m_layout->addWidget(rightHandle, 1, 2);*/
    
//     setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Expanding);
    
    if (m_area)
        dock(m_area, m_place);
}

DockWidget::~DockWidget()
{
}

void DockWidget::setWidget(QWidget *widget)
{
    m_widget = widget;
    if (m_widget)
    {
        m_widget->reparent(this, QPoint(0, 0), true);
        m_layout->addWidget(m_widget, 1, 1);
    }
}

QWidget *DockWidget::widget() const
{
    return m_widget;
}

void DockWidget::removeWidget()
{
    delete m_widget;
}

void DockWidget::setDockHeader(DockHeader *header)
{
    if (m_header)
        delete m_header;
    m_header = header;
    m_header->reparent(this, QPoint(0, 0), true);
    m_headerLayout->insertWidget(0, m_header);
}

void DockWidget::addDockHeaderButton(DockHeaderButton *button)
{
    button->reparent(this, QPoint(0, 0), true);
    m_headerLayout->insertWidget(1, button);
}

void DockWidget::detachOrDock()
{
    if (m_docked)
        undock();
    else
        dockBack();
}

void DockWidget::overlap()
{
    m_overlapped = !m_overlapped;
    emit overlapped(m_overlapped);
}

void DockWidget::closeDock()
{
    emit closed();
}

bool DockWidget::isDocked() const
{
    return m_docked;
}

bool DockWidget::isOverlapped() const
{
    return m_overlapped;
}

bool DockWidget::eventFilter(QObject */*watched*/, QEvent *e)
{
    if (e->type() == QEvent::MouseButtonPress)
    {
        startDrag();
        return true;
    }
    else if (e->type() == QEvent::MouseButtonRelease)
    {
        stopDrag();
        QPair<DockArea*, Place> areaInfo = findArea(e);
        DockArea *area = areaInfo.first;
        if (area && area != DockWidget::dockArea())
        {
            kdDebug() << "mouse release on dock area: " << area->name() << endl;
            dock(area, areaInfo.second);
        }
        return true;
    }
    else if (e->type() == QEvent::MouseMove && m_dragging)
    {
        QPair<DockArea*, Place> areaInfo = findArea(e);
        DockArea *area = areaInfo.first;
        if (area && area != DockWidget::dockArea())
        {
            kdDebug() << "mouse move on dock area" << endl;
            area->drawDockPlace(area->dockPlace(this));
        }
        return true;
    }
    return false;
}

void DockWidget::startDrag()
{
    qApp->setOverrideCursor(QCursor(Qt::SizeAllCursor));
    m_dragging = true;
}

void DockWidget::stopDrag()
{
    qApp->restoreOverrideCursor();
    m_dragging = false;
}

QPair<DockArea*, Place> DockWidget::findArea(QEvent *e)
{
    QMouseEvent *me = dynamic_cast<QMouseEvent*>(e);
    if (!me)
        return qMakePair((DockArea*)0, Place());
    
    QWidget *w = qApp->widgetAt(me->globalPos(), false);
    if (!w)
        return qMakePair((DockArea*)0, Place());
    
    DockArea *area = dynamic_cast<DockArea *>(w);
    if (!area)
    {
        QObjectList *list = w->queryList("Ideal::DockArea");
        if (list->count())
            area = dynamic_cast<DockArea *>(list->first());
        delete list;
    }
    QPoint p = me->globalPos();
    return qMakePair(area, area->placeHint(p));
}

void DockWidget::dock(DockArea *area, Place place)
{   
    saveConfig();
    area->placeDockWidget(this, place, dockInfo(area->name("dock area")));

    //FIXME: double check!!!
    if (m_area->isEmpty())
         delete m_area;
    
    m_area = area;
    m_place = place;
    m_docked = true;
    m_detachOrDockButton->show();
    m_detachOrDockButton->setType(DockHeaderButton::Detach);
    emit docked(m_docked);
}

void DockWidget::dockBack()
{
    if (m_oldArea)
        dock(m_oldArea, m_oldPlace);
}

void DockWidget::undock()
{
    DockArea *area = new DockArea(DockArea::Bottom, 0, name());
    area->placeDockWidget(this, Ideal::Bottom, dockInfo(name()));
    area->setMode(DockArea::DockContainer);
    area->show();
    
    m_oldArea = m_area;
    m_oldPlace = m_oldPlace;
    
    m_area = area;
    m_place = Ideal::Bottom;
    m_docked = false;
    m_detachOrDockButton->setType(DockHeaderButton::Dock);
    if (!m_oldArea)
        m_detachOrDockButton->hide();
    emit docked(m_docked);
}

DockArea *DockWidget::dockArea() const
{
    return m_area;
}

void DockWidget::initConfig()
{
    if (m_dockInfo.contains(" "))
        setGeometry(m_dockInfo[" "].geometry);
}

void DockWidget::saveConfig()
{
    DockInfo &info = m_dockInfo[m_area->name("dock area")];
    info.geometry = geometry();
//     info.layoutInfo.col = area->placeWidget();
}

DockInfo *DockWidget::dockInfo(const char *name)
{
    if (!m_dockInfo.contains(name))
    {
        m_dockInfo[name].geometry = geometry();
    }
    return &m_dockInfo[name];
}

}

#include "dockwidget.moc"
