/***************************************************************************
 *   Copyright (C) 2004 by Alexander Dymo                                  *
 *   adymo@kdevelop.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IDEALDOCKWIDGET_H
#define IDEALDOCKWIDGET_H

#include <qwidget.h>

#include "comdefs.h"

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;

namespace Ideal {

class DockArea;
class DockHeader;
class DockHeaderButton;
class DockResizeHandle;

/**Information about dock - position, size, layouting info.*/
struct DockInfo {
    DockInfo();
    
    QRect geometry;
    struct LayoutInfo {
        int row;
        int col;
    } layoutInfo;
    int dockPosition;
    
    QString write();
    
    static DockInfo read(const QString &str);
};

/**
@short Dock widget for ideal library.

Dock widgets can be standalone or in dock areas. If dock widget is in an area
then the area takes care about sizing and placement of a dock widget.
*/
class DockWidget: public QWidget {
    Q_OBJECT
public:
    /**Constructs floatable dock widget.*/
    DockWidget(const char *name);
    /**Constructs docked widget in the dock area @arg area and at the dock place @arg place.*/
    DockWidget(DockArea *area, Place place, const char *name);
    ~DockWidget();
    
    /**Sets the widget in the dock.*/
    void setWidget(QWidget *widget);
    /**Returns a widget in the dock.*/
    QWidget *widget() const;
    /**Removes the widget from the dock. This deletes the widget.*/
    void removeWidget();

    /**Queries a docked state of a dock.*/    
    bool isDocked() const;
    /**Queries an overlapped state of a dock.*/
    bool isOverlapped() const;
    
    /**Docks to the the dock area.
    @param place dock place in the area*/
    void dock(DockArea *area, Place place);
    /**Docks back to the previous dock area and position*/
    void dockBack();
    /**Undocks the widget.*/
    void undock();
    /**Returns the dock area.*/
    DockArea *dockArea() const;

    virtual bool eventFilter(QObject *watched, QEvent *e);
    
signals:
    /**Close button was pressed.*/
    void closed();
    /**Overlap button was pressed. @arg b is true if overlap mode is set.*/
    void overlapped(bool b);
    /**Dock button was pressed. @arg b is true if widget is docked.*/
    void docked(bool b);
    
public slots:
    /**Changes the docking state.*/
    virtual void detachOrDock();
    /**Changes the overlap state.*/
    virtual void overlap();
    /**Asks for closing a dock.*/
    virtual void closeDock();

protected:
    /**Sets the dock header.*/
    void setDockHeader(DockHeader *header);
    /**Adds the dock header button.*/
    void addDockHeaderButton(DockHeaderButton *button);
    
    void startDrag();
    void stopDrag();
    
    virtual void initConfig();
    virtual void saveConfig();
    
    QWidget *m_widget;
    DockHeader *m_header;
    DockHeaderButton *m_detachOrDockButton;
    DockHeaderButton *m_overlapButton;    
    DockHeaderButton *m_closeButton;    
    
    QHBoxLayout *m_headerLayout;
    QGridLayout *m_layout;
    
    DockResizeHandle *leftHandle, *rightHandle, *topHandle, *bottomHandle;

    //dock information on different areas, own dock size has " " key
    QMap<QString, DockInfo> m_dockInfo;
    DockInfo *dockInfo(const char *name);
    
private:
    void init();
    QPair<DockArea*, Place> findArea(QEvent *e);
    
    bool m_docked;
    bool m_overlapped;
    bool m_dragging;
    
    DockArea *m_area;
    Place m_place;
    
    DockArea *m_oldArea;
    Place m_oldPlace;
};

}

#endif
